#!/bin/sh
[ -z "${INC_JSHN}" ] && . ${BE_JSHN}/jshn.sh && INC_JSHN=1
IF_NAME="eth0"
MAC_PATTERN="HWaddr"
DEV_ROOT="/dev/root"
DEV_DATA="/dev/mmcblk0p4 /dev/mtdblock6 /dev/root"
barix_webui_conf_host() {
	IF_NAME="wlp3s0"
	MAC_PATTERN="ether"
	DEV_ROOT="/dev/sda2"
	DEV_DATA="/dev/sda3"
	return 0
}
do_list() {
	cat <<-EOF
	{
		"ping": {},
		"echo": {"msg": "String"},
		"status_get_all": {},
		"status_get_by_name": {"name": "String"},
	}
	EOF
}
status_elems="\
	mac_lan \
	hw_type \
	ipam_type \
	fsinfo_data \
	fsinfo_root \
	version_btl \
	version_kernel \
	version_fw \
	uptime \
	"
json_load_from_args() {
	local json="${1}"
	local args
	if [ -z "${json}" ]; then
		read args
		json="${args}"
	fi
	json_init
	json_load "${json}"
}
call_ping() {
	echo '{"msg": "Hello from Barix WebUI Backend!"}'
}
call_echo() {
	local msg
	json_load_from_args
	json_get_var msg "msg" || return $?
	echo "{\"echo-msg\": \"${msg}\"}"
}
get_field_value() {
	local list="$1"
	local field="$2"
	echo "${list}" | awk -v field="${field}" '{
		for (i = 1; i < NF; i++) {
			if ($i == field) {
				print $(i + 1);
				exit 0;
			}
		}
		exit 1;
	}' || return $?
}
mac_get() {
	local dev="$1"
	local field mac
	mac=$(ifconfig "${dev}") || return $?
	field="${MAC_PATTERN}"
	mac=$(echo "${mac}" | grep -w "${field}") || return $?
	mac=$(get_field_value "${mac}" "${field}") || return $?
	echo "${mac}"
}
fsinfo_get() {
	local dev="$1"
	local df_line params total used percent
	local fs_info fs_type fs_flags fs_status
	local val
	local grep_opts="-w"
	local grep_regex="^${dev}\|${dev}$"
	df_line=$(df -h) || return $?
	df_line=$(echo "${df_line}" | grep "${grep_opts}" "${grep_regex}") \
		|| return $?
	df_line=$(echo "${df_line}" | uniq) || return $?
	params=$(echo "${df_line}" | awk '{printf("%s %s %s", $2, $3, $5)}') \
		|| return $?
	read total used percent <<-EOF
	$(echo "${params}")
	EOF
	fs_info=$(cat /proc/mounts) || return $?
	fs_info=$(echo "${fs_info}" | grep "${grep_opts}" "${grep_regex}") \
		|| return $?
	fs_type=$(echo "${fs_info}" | awk '{print $3}') || return $?
	fs_flags=$(echo "${fs_info}" | awk '{print $4}') || return $?
	fs_status=$(echo "${fs_flags}" | awk -F',' '{print $1}') || return $?
	case "${fs_status}" in
		rw)
			fs_status="Read-Write"
			;;
		ro)
			fs_status="Read-Only"
			;;
	esac
	val="{\"total\": \"${total}\", \"used\": \"${used}\""
	val="${val}, \"percent\": \"${percent}\""
	val="${val}, \"type\": \"${fs_type}\", \"status\": \"${fs_status}\"}"
	echo "${val}"
}
fsinfo_dev() {
	local stor="$1"
	case "${stor}" in
		root)
			echo "${DEV_ROOT}"
			;;
		data)
			local try_devs="${DEV_DATA}"
			for dev in ${try_devs}; do
				cat /proc/mounts | grep -q -w "^${dev}" && {
					echo "${dev}"
					break
				}
			done
			;;
	esac
}
factory_info_json=$(cat <<EOF
{
	"HW_DEVICE":
	{
		"Product_ID": "2017.9999",
		"Product_Name": "Barix Instreamer Q398",
		"Production_Date": "23\/03\/2017",
		"Product_SN": "0123456789",
		"Lot_Number": "some number",
		"Legacy_HW_Type": "58",
		"Legacy_IPAM_Type": "7",
		"IPAM_Name": "QINO-398",
		"RTC_Calibration_Value": "-39",
		"LAN MAC Address": "",
		"WIFI MAC Address": "",
		"U-Boot Version": "Unknown U-Boot version",
		"Firmware Version": "1.04"
	}
}
EOF
)
factory_info_file=$(uci -q get factory_info.main.export_path) && {
	factory_info_json=$(cat "${factory_info_file}")
}
factory_info_get() {
	local type="$1"
	local name="$2"
	local data
	json_load "$factory_info_json"
	json_select "${type}" || return $?
	json_get_var data "${name}" || return $?
	echo "${data}"
}
uptime_get() {
	local uptime
	uptime=$(cat /proc/uptime) || return $?
	uptime=${uptime/.*/} || return $?
	echo "${uptime}"
}
status_get() {
	local elem="$1"
	local value
	local is_obj=0
	case "${elem}" in
		mac_lan)
			value=$(mac_get "${IF_NAME}") || return $?
			;;
		hw_type)
			value=$(factory_info_get "HW_DEVICE" "Legacy_HW_Type") \
				|| return $?
			;;
		ipam_type)
			value=$(factory_info_get "HW_DEVICE" "Legacy_IPAM_Type") \
				|| return $?
			;;
		fsinfo_data)
			value=$(fsinfo_get $(fsinfo_dev "data")) || return $?
			is_obj=1
			;;
		fsinfo_root)
			value=$(fsinfo_get $(fsinfo_dev "root")) || return $?
			is_obj=1
			;;
		version_btl)
			value=$(factory_info_get "HW_DEVICE" "U-Boot Version") \
				|| return $?
			;;
		version_kernel)
			value=$(uname -a) || return $?
			;;
		version_fw)
			value=$(factory_info_get "HW_DEVICE" "Firmware Version") \
				|| return $?
			;;
		uptime)
			value=$(uptime_get) || return $?
			;;
		*)
			return 1
			;;
	esac
	[ $is_obj -eq 0 ] && value="\"${value}\""
	echo "\"${elem}\": ${value}"
}
status_get_one() {
	local elem="$1"
	local data
	data=$(status_get "${elem}") || return $?
	echo "{${data}}"
}
call_status_get_all() {
	local resp="{"
	local i=0
	for elem in ${status_elems}; do
		[ ${i} -gt 0 ] && resp="${resp}, "
		resp="${resp}$(status_get ${elem})" || return $?
		i=$((${i}+1))
	done
	resp="${resp}}"
	echo "${resp}"
}
call_status_get_by_name() {
	local sign="$1"
	local name
	json_load_from_args "${sign}"
	json_get_var name "name" || return $?
	status_get_one "${name}" || return $?
}
do_call() {
	local proc="$1"
	local sign=""
	case "${proc}" in
		ping)
			call_ping
			;;
		echo)
			call_echo
			;;
		status_get_all)
			call_status_get_all
			;;
		status_get_by_name)
			ubus_sign_get "${post_body}" sign || return 2
			call_status_get_by_name "${sign}"
			;;
		*)
			return 1
			;;
	esac
}
# TEST MODE! When testing the Backend Shell Service handler for Barix WebUI on
if [ ${BE_TGT} -eq 0 ]; then
	barix_webui_conf_host
fi
cmd="$1"
case "${cmd}" in
	list)
		do_list
		;;
	call)
		proc="$2"
		do_call "${proc}"
		;;
esac
