#!/bin/sh
haserl_env_syslog() {
	local f="${FUNCNAME[0]}"
	slog_inf "${f}" "+-----"
	slog_inf "${f}" "| HASERLVER = '${HASERLVER}'"
	slog_inf "${f}" "| SESSIONID = '${SESSIONID}'"
	slog_inf "${f}" "| HASERL_ACCEPT_ALL = '${HASERL_ACCEPT_ALL}'"
	slog_inf "${f}" "| HASERL_SHELL = '${HASERL_SHELL}'"
	slog_inf "${f}" "| HASERL_UPLOAD_DIR = '${HASERL_UPLOAD_DIR}'"
	slog_inf "${f}" "| HASERL_UPLOAD_LIMIT = '${HASERL_UPLOAD_LIMIT}'"
	slog_inf "${f}" "+--"
	slog_inf "${f}" "| REQUEST_METHOD = '${REQUEST_METHOD}'"
	slog_inf "${f}" "| CONTENT_TYPE = '${CONTENT_TYPE}'"
	slog_inf "${f}" "| CONTENT_LENGTH = '${CONTENT_LENGTH}'"
	slog_inf "${f}" "| QUERY_STRING = '${QUERY_STRING}'"
	slog_inf "${f}" "+-----"
	return 0
}
haserl_var_get() {
	local name="${1}"
	local retv="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	utils_check_retv "${retv}" "rslt" || return $?
	if [ -z "${name}" ] || [ -z "${retv}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	eval "rslt=\${${name}}"
	eval "${retv}=\${rslt}"
	return 0
}
haserl_file_detect() {
	local retv="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local env
	local ptn
	local file
	utils_check_retv "${retv}" "ret" "env" "ptn" "file" || return $?
	utils_cmd_exists "grep" || return $?
	utils_cmd_exists "sed" || return $?
	if [ -z "${retv}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	env=$(export)
	ptn="HASERL_\([^_]*\)_path"
	file=$(grep -o "${ptn}" <<< "${env}" 2>/dev/null | \
		sed -e "s/${ptn}/\1/g" 2>/dev/null) || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Error '${ret}' looking for file no environment!"
		return ${ret}
	elif [ -z "${file}" ]; then
		jlog_err "${f}" "No file detected on environment!"
		return 1
	fi
	eval "${retv}=\${file}"
	return 0
}
haserl_file_import() {
	local file="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local fpath=""
	local fname=""
	local usign
	local upost=""
	if [ -z "${file}" ]; then
		jlog_err "${f}" "Missing mandatory arguments!"
		return 1
	fi
	if [ "${file}" != "instreamer4" ]; then
		jlog_err "${f}" "Only file 'instreamer4' is supported!"
		return 2
	fi
	haserl_var_get "HASERL_${file}_path" fpath || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Error '${ret}' retrieving '${file}' file path!"
		return ${ret}
	fi
	haserl_var_get "FORM_${file}_name" fname || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Error '${ret}' retrieving '${file}' file name!"
		return ${ret}
	fi
	slog_inf "${f}" "+-----"
	slog_inf "${f}" "| HASERL_${file}_path = '${fpath}'"
	slog_inf "${f}" "| FORM_${file}_name = '${fname}'"
	slog_inf "${f}" "+-----"
	usign='{"filepath":"'${fpath}'","filename":"'${fname}'"}'
	ubus_build_post "${file}" "import" "${usign}" upost || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Error '${ret}' building the uBus request!"
		return ${ret}
	fi
	post_body="${upost}"
	do_ubus "${upost}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${S}" "Error '${ret}' processing uBus request!"
	fi
	return 3
}
INC_HARSERL=1
