#!/bin/sh
_uci_set_add_array() {
	local blob="${1}"
	local vals="${2}"
	local name="${3}"
	local file="${4}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local pkg sec val
	local len=0
	local idx
	IFS=":" read -r pkg sec val <<< "${blob}" || return 1
	jq_arr_len_get "${vals}" "${name}" len || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get length of '${name}' array!"
		return ${ret}
	fi
	for idx in $(seq 0 $((${len}-1))); do
		val=""
		jq_arr_idx_get "${vals}" "${name}" "${idx}" val || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Could not get index '${idx}' of '${name}' array!"
			return ${ret}
		fi
		echo "add_list ${pkg}.${sec}.${name}=${val}" >> "${file}"
	done
	return 0
}
_uci_set_add_value() {
	local blob="${1}"
	local vals="${2}"
	local name="${3}"
	local file="${4}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local pkg sec val
	IFS=":" read -r pkg sec val <<< "${blob}" || return 1
	val=""
	jq_elem_get "${vals}" "${name}" val || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get '${name}' value!"
		return ${ret}
	fi
	echo "set ${pkg}.${sec}.${name}=${val}" >> "${file}"
	return 0
}
_uci_set() {
	local blob="${1}"
	local vals="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local uci=""
	local uci_file=""
	local keys=""
	local key
	local typ=""
	utils_uci_cmd uci || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get UCI tool's command line!"
		return ${ret}
	fi
	utils_mktemp_file uci_file || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not create temporary file for UCI commands!"
		return ${ret}
	fi
	jq_keys_get "${vals}" keys || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get keys from 'values' object!"
		utils_rm_file "${uci_file}"
		return ${ret}
	fi
	for key in ${keys}; do
		jq_typ_get "${vals}" "${key}" typ || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Could not get type of '${key}'!"
			utils_rm_file "${uci_file}"
			return ${ret}
		fi
		case "${typ}" in
		"array")
			_uci_set_add_array "${blob}" "${vals}" "${key}" \
				"${uci_file}" || ret=$?
			if [ ${ret} -ne 0 ]; then
				jlog_err ${f} "Could not add '${key}' array!"
				utils_rm_file "${uci_file}"
				return ${ret}
			fi
			;;
		*)
			_uci_set_add_value "${blob}" "${vals}" "${key}" \
				"${uci_file}" || ret=$?
			if [ ${ret} -ne 0 ]; then
				jlog_err ${f} "Could not add '${key}' element!"
				utils_rm_file "${uci_file}"
				return ${ret}
			fi
			;;
		esac
	done
	${uci} -f "${uci_file}" batch || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error in 'uci batch' command on '"${uci_file}"'!"
		utils_rm_file "${uci_file}"
		return ${ret}
	fi
	utils_rm_file "${uci_file}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' removing '${uci_file}'!"
		return ${ret}
	fi
	return 0
}
_uci_set_validate_array() {
	local vals="${1}"
	local name="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local array=""
	local types=""
	local typ
	jq_elem_get "${vals}" "${name}" array || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get '${name}' array!"
		return ${ret}
	fi
	jq_types_get "${array}" types || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get array's types!"
		return ${ret}
	fi
	if [ -z "${types}" ]; then
		jlog_err ${f} "Array cannot be empty!"
		return 1
	fi
	for typ in ${types}; do
		if [ "${typ}" = "object" ] || [ "${typ}" = "array" ]; then
			jlog_err ${f} "Cannot nest JSON objects!"
			return 2
		fi
	done
	return 0
}
_uci_set_validate_values() {
	local vals="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local keys=""
	local key
	local typ=""
	jq_keys_get "${vals}" keys || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get keys from 'values' object!"
		return ${ret}
	fi
	if [ -z "${keys}" ]; then
		jlog_err ${f} "Object 'values' cannot be empty!"
		return 1
	fi
	for key in ${keys}; do
		jq_typ_get "${vals}" "${key}" typ || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Could not get type of '${key}'!"
			return ${ret}
		fi
		case "${typ}" in
		"object")
			jlog_err ${f} "Objects ('${key}') within 'values' are not allowed!"
			return 2
			;;
		"array")
			_uci_set_validate_array "${vals}" "${key}" || ret=$?
			if [ ${ret} -ne 0 ]; then
				jlog_err ${f} "Array '${key}' within 'values' is invalid!"
				return 3
			fi
			;;
		*)
			;;
		esac
	done
	return 0
}
_uci_set_validate_blob() {
	local blob="${1}"
	local f="${FUNCNAME[0]}"
	local pkg sec vals
	IFS=":" read -r pkg sec vals <<< "${blob}" || return 1
	if [ -z "${pkg}" ]; then
		jlog_err ${f} "Config must be specified!"
		return 1
	elif [ -z "${sec}" ]; then
		jlog_err ${f} "Section must be specified!"
		return 2
	elif [ -z "${vals}" ]; then
		jlog_err ${f} "Values must be specified!"
		return 3
	else
		:
	fi
	return 0
}
_uci_set_load() {
	local sign="${1}"
	local retv1="${2}"
	local retv2="${3}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local keys=""
	local key
	local pkg=""
	local sec=""
	local vals=""
	local blob
	utils_check_retv "${retv1}" "sign" "keys" "key" "pkg" "sec" "vals" \
		"blob" || return $?
	utils_check_retv "${retv2}" "sign" "keys" "key" "pkg" "sec" "vals" \
		"blob" || return $?
	jq_keys_get "${sign}" keys || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get signature's keys!"
		return ${ret}
	fi
	for key in ${keys}; do
		case "${key}" in
		"config")
			jq_str_get "${sign}" "${key}" pkg || ret=$?
			if [ ${ret} -ne 0 ]; then
				jlog_err ${f} "Could not get 'config' value!"
				return ${ret}
			fi
			;;
		"section")
			jq_str_get "${sign}" "${key}" sec || ret=$?
			if [ ${ret} -ne 0 ]; then
				jlog_err ${f} "Could not get 'section' value!"
				return ${ret}
			fi
			;;
		"values")
			jq_obj_get "${sign}" "${key}" vals || ret=$?
			if [ ${ret} -ne 0 ]; then
				jlog_err ${f} "Could not get 'values' object!"
				return ${ret}
			fi
			;;
		*)
			jlog_err ${f} "Key '${key}' is invalid!"
			return 1
			;;
		esac
	done
	for tok in ${pkg} ${sec}; do
		if [ "${tok##*:}" != "${tok}" ]; then
			jlog_err ${f} "Token '${tok}' cannot have semicolons!"
			return 2
		fi
	done
	blob="${pkg}:${sec}"
	[ -n "${vals}" ] && blob="${blob}:1"
	eval "${retv1}=\${blob}"
	eval "${retv2}=\${vals}"
	return 0
}
uci_set() {
	local sign="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local sblob=""
	local svals=""
	if [ -z "${sign}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	_uci_set_load "${sign}" sblob svals || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error loading 'signature'!"
		return ${ret}
	fi
	_uci_set_validate_blob "${sblob}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Request's 'signature' operations are not valid!"
		return ${ret}
	fi
	_uci_set_validate_values "${svals}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Request's 'signature' values are not valid!"
		return ${ret}
	fi
	_uci_set "${sblob}" "${svals}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error setting UCI configuration!"
		return ${ret}
	fi
	return 0
}
INC_UCI_SET=1
