#!/bin/sh
_app_pgrep_map_name() {
	local app="${1}"
	local retv="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local pname
	utils_check_retv "${retv}" "app" "pname" || return $?
	case "${app}" in
	"audiostreamer")
		pname="Timer"
		;;
	*)
		pname="${app}"
		;;
	esac
	eval "${retv}=\${pname}"
	return 0
}
app_is_running() {
	local app="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local mapp=""
	local rslt
	utils_cmd_exists "pgrep" || return $?
	if [ -z "${app}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 3
	fi
	_app_pgrep_map_name "${app}" mapp || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' mapping '${app}' to feed 'pgrep'!"
		return 4
	fi
	rslt=$(pgrep "${mapp}" 2>&1) || ret=$?
	if [ ${ret} -ne 0 ]; then
		if [ -n "${rslt}" ]; then
			jlog_err ${f} "Error '${ret}' in 'pgrep' command!"
			jlog_err ${f} "${rslt}"
			return 2
		fi
		return 1
	fi
	return 0
}
app_start() {
	local app="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local rslt
	if [ -z "${app}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	app_is_running "${app}"
	rslt=$?
	if [ ${rslt} -eq 0 ]; then
		slog_inf ${f} "'${app}' is already running"
	elif [ ${rslt} -eq 1 ]; then
		slog_inf ${f} "Starting '${app}' init script"
		/etc/init.d/"${app}" start &>/dev/null || ret=$?
	else
		jlog_err ${f} "Could not check if '${app}' is running!"
		return ${rslt}
	fi
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' starting '${app}' init script!"
		return ${ret}
	fi
	return 0
}
app_stop() {
	local app="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	if [ -z "${app}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	slog_inf ${f} "Stopping '${app}' init script"
	/etc/init.d/"${app}" stop &>/dev/null || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' stopping '${app}' init script!"
		return ${ret}
	fi
	return 0
}
